## Rolling Upgrades

The approach upgrades each etcd node one by one, and eventually gives you a etcd cluster running on the new version.

Zero downtime is achieved through rolling upgrades if the etcd cluster is healthy. For example, you have a 3-member cluster and start to upgrade it. When stopping one member for upgrade, the cluster has two members alive and is still able to make progress because its majority works. Because the member is upgraded one by one, the cluster keeps working in the whole process.

New feature is disabled during the rolling upgrade process, and is activated automatically after all nodes were upgraded. Internally, etcd members negotiate with each other to determine etcd cluster version, which controls the availability of new features.

Rolling upgrades is supported for one further minor version due to [etcd backwards compatibility](#backwards-compatible-schemas). Moreover, etcd doesn't support roll back, and cannot downgrade to early version. If you may go back to early version, please backup etcd data directories in advance.

### Backwards Compatible Schemas

All modules in etcd guarantee that they could work across one minor version back. For example, upgrades from v2.0 to v2.1 are tested and supported, while upgrades from v2.0 to v2.2 are not. Moreover, if v2.3 is the last minor version in v2, it supports to be rolling upgraded to v3.0.

### Example Workflow

The workflow on how to rolling upgrade one 3-member etcd cluster:

1. check the cluster is healthy:

```
$ etcdctl cluster-health
```

2. check the cluster version is correct:

```
$ curl http://127.0.0.1:4001/version
```

3. stop etcd process

4. back up data directory:

```
$ tar -czf etcd_backup.tar.gz ${ETCD_DATA_DIR}
```

5. upgrade etcd binary

6. start etcd using the old command line

7. verify the cluster is healthy

```
$ etcdctl cluster-health
```

8. repeat the process from step 3 for remaining members in the cluster

9. check the cluster version is changed as expected:

```
$ curl http://127.0.0.1:4001/version
```
